package core

import (
	"fmt"
	"math"

	"git.urbach.dev/cli/q/src/asm"
	"git.urbach.dev/cli/q/src/errors"
	"git.urbach.dev/cli/q/src/eval"
	"git.urbach.dev/cli/q/src/expression"
	"git.urbach.dev/cli/q/src/types"
)

var _len = Function{OutputTypes: []types.Type{types.AnyInt}}

// CompileLen returns the length of a slice.
func (f *Function) CompileLen(root *expression.Expression) error {
	value, err := f.Evaluate(root.Children[1])

	if err != nil {
		return err
	}

	if !types.Is(value.Type(), types.AnyArray) {
		return errors.New(&errors.TypeMismatch{Encountered: value.Type().Name(), Expected: types.AnyArray.Name(), ParameterName: "array"}, f.File, root.Children[1].Token.Position)
	}

	memory := asm.Memory{
		Offset:         -8,
		OffsetRegister: math.MaxUint8,
		Length:         8,
	}

	output := f.CPU.Output[0]
	f.SaveRegister(output)

	switch value := value.(type) {
	case *eval.Register:
		memory.Base = value.Register
	case *eval.Label:
		f.RegisterLabel(asm.MOVE, output, value.Label)
		memory.Base = output
	default:
		panic(fmt.Errorf("%s: not implemented: %v", f.UniqueName, value))
	}

	f.MemoryRegister(asm.LOAD, memory, output)
	return nil
}
